/**
 * \file PlayerKernel.cpp
 *
 * Implementation of the PlayerKernel class
 */

#include "Player/PlayerKernel.h"
#include "Common/Interfaces/MsgProcess.h"
#include "Common/Interfaces/Commands.h"
#include "Player/PlayerKernel.h"
#include "Common/Tasks.h"
#include "Common/Support/Debug.h"
#include <stdio.h>
#include <math.h>
#include <string.h>

#ifdef WIN32
#include <windows.h>
#endif

PlayerKernel::PlayerKernel(
	const char* name /* = "PlayerKernel"*/,
	const char* desc /* = "PlayerKernel module"*/)
	: Module(name, desc, "v0.2.000")
{
	// Create new world
	world = new World();
	
	// Initialize the world
	world->me->pos.SetX(-37);
	world->state->SetPlayMode(PM_BeforeKickOff);

	// Initialize pointers to last sense body and last visual info
	lastSb = NULL;
	lastVi = NULL;
	lastViTime = 0;
	lastSbTime = 0;
	
	// Initialize synchronization
	synchMode = Synch_Normal;
	nextSynchMode = Synch_Normal;
	decisionWasMade = true;

	// Initialize the feedback data to zeros
	feedback = new FEEDBACK_DATA;
	memset((void*) feedback, 0, sizeof(FEEDBACK_DATA));

	// Initialize command counts to zeros
	counts = new COMMAND_COUNTS;
	memset((void*) counts, 0, sizeof(COMMAND_COUNTS));

	Main::GetInstance()->RegisterModule(this, MEMORY_TASK);
	Main::GetInstance()->RegisterModule(this, SYNCHRONIZATION_TASK);
	World::BindInterfaces();
}

Interface* PlayerKernel::GetInterface(int task) const
{
	if(task == SYNCHRONIZATION_TASK) return (SynchronizationInterface*) this;
	else if(task == MEMORY_TASK) return (MemoryInterface*) this;
	else return NULL;
}

void PlayerKernel::OnInit(RealSide realSide, int uniform, PlayMode mode)
{
	myside = realSide;
	// Overloaded in PlayerSkills
	TXTN("Connected, %s side, uniform %i", realSide == LEFT ? "left" : "right", uniform);


#ifdef WIN32
	char str[20];
	sprintf(str,"#%d", uniform);
	SetConsoleTitle(str);
#endif

	// Set up side swapping
	CommandsInterface *commandGen = 
		(CommandsInterface *) Main::GetInstance()->GetInterface(COMMANDS_TASK);
	if(commandGen != NULL) commandGen->SetRealSide(realSide);
	
	MsgProcessInterface *parser = 
		(MsgProcessInterface *) Main::GetInstance()->GetInterface(MESSAGE_PROCESS_TASK);
	if(parser != NULL) parser->SetRealSide(realSide);
	
	MsgProcessInterface *clParser = 
		(MsgProcessInterface *) Main::GetInstance()->GetInterface(CLANG_MESSAGE_PROCESS_TASK);
	if(clParser != NULL) clParser->SetRealSide(realSide);

	// This is the position set by server to a newly connected player
	world->me->pos = Point(- uniform * 3.f, -37);
	world->me->SetUniform(uniform);
	world->state->SetPlayMode(mode);
}

void PlayerKernel::OnChangeMode(int time, PlayMode mode)
{
	// Overloaded in PlayerTactics
	PlayMode currMode = world->state->GetPlayMode();

	if((currMode == PM_BeforeKickOff && mode == PM_KickOff_Our) ||
		(currMode == PM_BeforeKickOff && mode == PM_KickOff_Their) ||
		(currMode == PM_AfterGoal_Our && mode == PM_KickOff_Their) ||
		(currMode == PM_AfterGoal_Their && mode == PM_KickOff_Our))
	{
		// We now surely know ball position, it is in the middle
		world->ball->pos = Point(0, 0);
		world->ball->vel = RectVector(0, 0);
		world->ball->SetAge(0);
		world->ball->SetValid();
	}

	world->state->SetPlayMode(mode);
}

void PlayerKernel::OnHear(int time, float direction, bool ourTeam, int uniform, const char* msg)
{
	// Overloaded in PlayerSkills
}

void PlayerKernel::OnScore(int time, int our, int their)
{
	world->state->SetScore(our, their);
}

void PlayerKernel::OnSenseBody(const SENSE_BODY* sb)
{
	static int sbCount = 0;
	sbCount++;

	PlayMode pm = world->state->GetPlayMode();
	if(sbCount % 3 != 0 &&
	   (pm == PM_Offside_Our || pm == PM_Offside_Their ||
		pm == PM_CatchFault_Our || pm == PM_CatchFault_Their ||
		pm == PM_BackPass_Our || pm == PM_BackPass_Their ||
		pm == PM_FreeKickFault_Our || pm == PM_FreeKickFault_Their
	   )
	  )
	{
		world->state->SetSimTime(world->state->GetSimTime() + 1);
		nextSynchMode = Synch_Normal;
	}

	if(world->state->IsTimeStoped() || sb->time != world->state->GetSimTime())
	{
		// Sense body after simulation time change

		// Save my previous position
		Point prevPos = world->me->pos;

		// Apply all commands I have performed since last sense body
		world->ApplyFeedback(feedback);
		
		// Simulate all object movement
		world->Step();
 
		// Save this sense body data pointer and time
		lastSb = sb;
		lastSbTime = lastSb->time;

		// Synchronization mode
		synchMode = nextSynchMode;

		// Change my state according to sense body data
		world->me->SetStamina(lastSb->stamina);
		world->me->SetEffort(lastSb->effort);
		world->me->SetNeckDir(DegAngle(lastSb->headAngle));
		world->me->SetViewWidth(lastSb->viewWidth);
		
		// Store command counts
		memcpy((void*) counts, (void*) &lastSb->counts, sizeof(COMMAND_COUNTS));

		// Do we have the visual information too?
		if(lastVi != NULL)
		{
			if(lastVi->time > world->state->GetSimTime())
			{
				// We got the VI sooner then SB - possibly an old server
				world->state->SetSimTime(lastSb->time);
				world->ProcessInfo(lastSb, lastVi);
				lastVi = NULL;

				// Decision making
				Act();
				decisionWasMade = true;
			}
		}
		else
		{
			// No VI yet
			world->state->SetSimTime(lastSb->time);
			
			// Set my speed
			Angle angle = world->me->GetAbsDir() + world->me->GetNeckDir() + DegAngle(lastSb->speedDirection);
			world->me->vel = PolarVector(lastSb->speedSize, angle);
		
			// Set my position
			world->me->pos = prevPos + world->me->vel / world->me->type->player_decay;
		
			// In Normal mode start making decision
			if(synchMode == Synch_Normal)
			{
				Act();
				decisionWasMade = true;
			}
			else
			{
				decisionWasMade = false;
			}
		}
	}
	else
	{
		// Sense body data, but no change in simulation time
		lastSb = sb;

		// Change world according to SB
		world->me->SetStamina(lastSb->stamina);
		world->me->SetEffort(lastSb->effort);
		world->me->SetNeckDir(DegAngle(lastSb->headAngle));
		world->me->SetViewWidth(lastSb->viewWidth);
		
		// Store command counts
		memcpy((void*) counts, (void*) &lastSb->counts, sizeof(COMMAND_COUNTS));

		Angle angle = world->me->GetAbsDir() + world->me->GetNeckDir() + DegAngle(lastSb->speedDirection);
		world->me->vel = PolarVector(lastSb->speedSize, angle);
	}
}

/*static void VIDUMP(const VISUAL_INFO* vi)
{
	int i;
		
	DBGN("--> VI DUMP start");

	DBGN("  LANDMARKS:");
	for(i = 0; i < vi->landmarkCnt; i++)
	{
		DBGN("    type %i dist %3.2f dir %3.2f distch %3.2f dirch %3.2f", vi->landmarks[i].type, vi->landmarks[i].dist, vi->landmarks[i].dir, vi->landmarks[i].distChange, vi->landmarks[i].dirChange);
	}

	DBGN("  LINES:");
	for(i = 0; i < vi->lineCnt; i++)
	{
		DBGN("    type %i dist %3.2f dir %3.2f", vi->lines[i].type, vi->lines[i].dist, vi->lines[i].dir);
	}

	DBGN("<-- VI DUMP end");
}*/

void PlayerKernel::OnSee(const VISUAL_INFO* vi)
{
//	VIDUMP(vi);

	lastVi = vi;
	lastViTime = lastVi->time;

	// If we do not have the SB, do nothing
	if(lastSb == 0) return;

	// If this is the first VI in a new cycle, wait for SB
	if(lastVi->time > lastSb->time) return;

	// Change world without making decisions (Normal mode)
	world->ProcessInfo(lastSb, lastVi);
	if(synchMode != Synch_Normal && !decisionWasMade)
	{
		Act();
		decisionWasMade = true;
	}

	lastVi = NULL;
}


void PlayerKernel::OnServerParam(const SERVER_PARAMS* params)
{
	world->params->ProcessServerParams(params);
}

void PlayerKernel::OnPlayerParam(const PLAYER_PARAMS* params)
{
	world->params->ProcessPlayerParams(params);
}

void PlayerKernel::OnPlayerType(int id, const PLAYER_TYPE* type)
{
	world->params->ProcessPlayerType(id, type);
}

void PlayerKernel::OnExecKick(float power, float direction)
{
	if(power > world->params->server->maxpower)
	{
		power = world->params->server->maxpower;
	}
	else if(power < world->params->server->minpower)
	{
		power = world->params->server->minpower;
	}

	if(feedback->lastKickPower == 0)
	{
		feedback->lastKickPower = power;
		feedback->lastKickDir = direction;
	}
}

void PlayerKernel::OnExecTurn(float moment)
{
	if(moment > world->params->server->maxmoment)
	{
		moment = world->params->server->maxmoment;
	}
	else if(moment < world->params->server->minmoment)
	{
		moment = world->params->server->minmoment;
	}

	if(feedback->lastTurn == 0) feedback->lastTurn = moment;
}

void PlayerKernel::OnExecDash(float power)
{
	if(power > world->params->server->maxpower)
	{
		power = world->params->server->maxpower;
	}
	else if(power < world->params->server->minpower)
	{
		power = world->params->server->minpower;
	}

	if(power >= 0)
	{
		if(power > world->me->GetStamina() + world->me->type->extra_stamina)
		{
			power = world->me->GetStamina() + world->me->type->extra_stamina;
		}
	}
	else
	{
		// Power is spended twice that much
		if(fabsf(power * 2) > world->me->GetStamina() + world->me->type->extra_stamina)
		{
			power = (world->me->GetStamina() + world->me->type->extra_stamina) / -2.0f;
		}
	}

	if(feedback->lastPower == 0) feedback->lastPower = power;
}

void PlayerKernel::OnExecNeck(int angle)
{
	if(angle > world->params->server->maxneckmoment)
	{
		angle = int(world->params->server->maxneckmoment);
	}
	else if(angle < world->params->server->minneckmoment)
	{
		angle = int(world->params->server->minneckmoment);
	}

	if(feedback->lastNeck == 0) feedback->lastNeck = angle;
}

void PlayerKernel::OnChangePlayerType(int uniform, int type)
{
	world->SetPlayerType(OUR, uniform, type);
	if (uniform==world->me->GetUniform())
	{
		world->me->SetSide(OUR);
		world->me->SetType(type);
	}
}

// GP
void PlayerKernel::OnFreeform(const char* msg)
{
   // TODO
   // take apropriate action based on recieved freeform message
   ;
}

PlayerKernel::~PlayerKernel()
{
	if(counts != NULL) delete counts;
	if(feedback != NULL) delete feedback;
	if(world != NULL) delete world;
}
